#include <sourcemod>
#include <sdktools>

#define PLUGIN_VERSION "1.0rc"

public Plugin:myinfo =
{
	name = "DoD ExplosionGibs",
	author = "FeuerSturm",
	description = "Players explode on Grenade/Bazooka impact!",
	version = PLUGIN_VERSION,
	url = "http://community.dodsourceplugins.net"
}

new Handle:ExpGibsStatus = INVALID_HANDLE
new Handle:ExpGibsMinDmg = INVALID_HANDLE
new Handle:ExpGibsLifeTime = INVALID_HANDLE
new Handle:ExpGibsBloodFountain = INVALID_HANDLE
new bool:ExplodePlayer[MAXPLAYERS+1]

#define MAXBONES 8

new String:BoneModel[MAXBONES][] =
{
	"models/gibs/hgibs.mdl",
	"models/gibs/hgibs_rib.mdl", "models/gibs/hgibs_rib.mdl", "models/gibs/hgibs_rib.mdl", "models/gibs/hgibs_rib.mdl",
	"models/gibs/hgibs_scapula.mdl", "models/gibs/hgibs_scapula.mdl",
	"models/gibs/hgibs_spine.mdl"
}

new String:BloodParticles[2][] =
{
	"UnlitGeneric\r{\r\"$translucent\" 1\r\"$basetexture\" \"Decals/blood_gunshot_decal\"\r\"$vertexcolor\" 1\r}",
	"materials/particle/particledefault.vmt"
}

new Handle:ParticleFile = INVALID_HANDLE

public OnPluginStart()
{
	ExpGibsStatus = CreateConVar("dod_explosiongibs_status", "1", "<1/0> = enable/disable exploding players",_, true, 0.0, true, 1.0)
	ExpGibsMinDmg = CreateConVar("dod_explosiongibs_mindamage", "75", "<#> = set minimum of damage to explode players",_, true, 1.0, true, 100.0)
	ExpGibsLifeTime = CreateConVar("dod_explosiongibs_bonelifetime", "30", "<#> = set time in seconds bones stay on the map",_, true, 5.0, true, 120.0)
	ExpGibsBloodFountain = CreateConVar("dod_explosiongibs_bloodfountain", "1", "<1/0> = enable/disable blood fountain while exploding",_, true, 0.0, true, 1.0)
	HookEventEx("dod_stats_player_damage", OnPlayerDamage, EventHookMode_Post)
	HookEventEx("dod_stats_player_killed", OnPlayerDeath, EventHookMode_Post)
	HookEventEx("player_spawn", OnPlayerSpawn, EventHookMode_Post)
	if(!FileExists(BloodParticles[1], true))
	{
		ParticleFile = OpenFile(BloodParticles[1], "a")
		if(ParticleFile != INVALID_HANDLE)
		{
			WriteFileString(ParticleFile, BloodParticles[0], false)
			CloseHandle(ParticleFile)
		}
	}
	AddFileToDownloadsTable(BloodParticles[1])
}

public OnMapStart()
{
	for(new i = 0; i < MAXBONES; i++)
	{
		if(!IsModelPrecached(BoneModel[i]))
		{
			PrecacheModel(BoneModel[i], true)
		}
	}
	PrecacheModel(BloodParticles[1], true)
}

public OnClientPostAdminCheck(client)
{
	ExplodePlayer[client] = false
}

public Action:OnPlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"))
	if(IsClientInGame(client) && IsPlayerAlive(client))
	{
		ExplodePlayer[client] = false
		return Plugin_Continue
	}
	return Plugin_Continue
}

public Action:OnPlayerDamage(Handle:event, const String:name[], bool:dontBroadcast)
{
	if(GetConVarInt(ExpGibsStatus) == 0)
	{
		return Plugin_Continue
	}
	new victim = GetClientOfUserId(GetEventInt(event, "victim"))
	new attacker = GetClientOfUserId(GetEventInt(event, "attacker"))
	new weapon = GetEventInt(event, "weapon")
	new damage = GetEventInt(event, "damage")
	if(victim == 0 || attacker == 0 || weapon < 17 || weapon > 28 || GetClientHealth(victim) > 0 || damage < GetConVarInt(ExpGibsMinDmg))
	{
		ExplodePlayer[victim] = false
		return Plugin_Continue
	}
	ExplodePlayer[victim] = true
	return Plugin_Continue
}

public Action:OnPlayerDeath(Handle:event, const String:name[], bool:dontBroadcast)
{
	new victim = GetClientOfUserId(GetEventInt(event, "victim"))
	if(GetConVarInt(ExpGibsStatus) == 0 || !ExplodePlayer[victim])
	{
		return Plugin_Continue
	}
	new Float:DeathOrigin[3]
	GetClientAbsOrigin(victim, DeathOrigin)
	new ShowBloodFount = GetConVarInt(ExpGibsBloodFountain)
	for(new i = 0; i < MAXBONES; i++)
	{
		PlayerExplode(DeathOrigin, BoneModel[i])
		if(ShowBloodFount == 1)
		{
			BloodFountain(victim)
		}
	}
	RemoveRagdoll(victim)
	return Plugin_Continue
}

PlayerExplode(Float:DeathOrigin[3], String:BoneMdl[])
{
	new Bone = CreateEntityByName("prop_physics")
	DispatchKeyValue(Bone, "model", BoneMdl)
	SetEntProp(Bone, Prop_Send, "m_CollisionGroup", 1)
	DispatchSpawn(Bone)
	DeathOrigin[0] += GetRandomFloat(1.0, 3.0)
	DeathOrigin[1] += GetRandomFloat(1.0, 3.0)
	DeathOrigin[2] += GetRandomFloat(10.0, 15.0)
	new Float:Velocity[3]
	Velocity[2] = GetRandomFloat(1.0, 5.0)
	new Float:Angles[3]
	Angles[2] = GetRandomFloat(25.0, 80.0)
	TeleportEntity(Bone, DeathOrigin, Angles, Velocity)
	CreateTimer(GetConVarFloat(ExpGibsLifeTime), RemoveBone, Bone, TIMER_FLAG_NO_MAPCHANGE)
}

BloodFountain(victim)
{
	new BloodSplash = CreateEntityByName("env_blood")
	DispatchSpawn(BloodSplash)
	DispatchKeyValue(BloodSplash, "color", "0")
	DispatchKeyValue(BloodSplash, "amount", "50000.0")
	DispatchKeyValue(BloodSplash, "spawnflags", "15")
	AcceptEntityInput(BloodSplash, "EmitBlood", victim)
	RemoveEdict(BloodSplash)
}

public Action:RemoveBone(Handle:Timer, any:Bone)
{
	if(IsValidEdict(Bone))
	{
		RemoveEdict(Bone)
	}
	return Plugin_Handled
}

RemoveRagdoll(victim)
{
	new Ragdoll = GetEntPropEnt(victim, Prop_Send, "m_hRagdoll")
	if(IsValidEdict(Ragdoll))
	{
		RemoveEdict(Ragdoll)
	}
}
